;; Generated package description from cursory.el  -*- no-byte-compile: t -*-
(define-package "cursory" "0.1.4.0.20220629.54451" "Manage cursor styles using presets" '((emacs "27.1")) :commit "39c252b001f8dde1cae18274dd9cbbc911fbb68e" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Cursory Development" . "~protesilaos/cursory@lists.sr.ht") :keywords '("convenience" "cursor") :url "https://git.sr.ht/~protesilaos/cursory")
